@@ -1,26 +1,33 @@
 #' Variable Table Function
 #'
-#' This function will output a descriptive variable table either to the console or as an HTML file that can be viewed continuously while working with data. vt() is the same thing but requires fewer key presses to type.
+#' This function will output a descriptive variable table either to the console or as an HTML file that can be viewed continuously while working with data. \code{vt()} is the same thing but requires fewer key presses to type.
 #'
 #' Outputting the variable table as a help file will make it easy to search through variable names or labels, or to refer to information about the variables easily.
 #'
-#' This function is in a similar spirit to promptData(), but focuses on variable documentation rather than dataset documentation.
+#' This function is in a similar spirit to \code{promptData()}, but focuses on variable documentation rather than dataset documentation.
 #'
-#' @param data Data set; accepts any format with column names. If variable labels are set with the haven package, set_label() from sjlabelled, or label() from Hmisc, vtable will extract them automatically.
-#' @param out Determines where the completed table is sent. Set to "browser" to open HTML file in browser using browseURL(), "viewer" to open in RStudio viewer using viewer(), if available. Use "htmlreturn" to return the HTML code to R, or "return" to return the completed variable table to R in data frame form. Defaults to "viewer" if RStudio is running and "browser" if it isn't.
-#' @param file Saves the completed variable table file to HTML with this filepath. May be combined with any value of out.
-#' @param labels Variable labels. labels will accept three formats: (1) A vector of the same length as the number of variables in the data, in the same order as the variables in the data set, (2) A matrix or data frame with two columns and more than one row, where the first column contains variable names (in any order) and the second contains labels, or (3) A matrix or data frame where the column names (in any order) contain variable names and the first row contains labels. Setting the labels parameter will override any variable labels already in the data. Set to "omit" if the data set has embedded labels but you don't want any labels in the table.
-#' @param class Set to TRUE to include variable classes in the variable table. Defaults to TRUE.
-#' @param values Set to TRUE to include the range of values of each variable: min and max for numeric variables, list of factors for factor or ordered variables, and 'TRUE FALSE' for logicals. values will detect and use value labels set by the sjlabelled or haven packages, as long as every value is labelled. Defaults to TRUE.
-#' @param missing Set to TRUE to include the number of NAs in the variable. Defaults to FALSE.
-#' @param index Set to TRUE to include the index number of the column with the variable name. Defaults to FALSE.
-#' @param factor.limit Sets maximum number of factors that will be included if values = TRUE. Set to 0 for no limit. Defaults to 5.
-#' @param char.values Set to TRUE to include values of character variables as though they were factors, if values = TRUE. Or, set to a character vector of variable names to list values of only those character variables. Defaults to FALSE. Has no effect if values = FALSE.
+#' If you would like to include a \code{sumtable} in an RMarkdown document, use \code{out="latexfrag"} if outputting to LaTeX (note this will just capture the table itself, not the header information). If outputting to another format, use \code{out="return"}, and send the result to \code{knitr::kable()}, perhaps followed by \code{kableExtra::kable_styling()} for styling control. Alternately, if you want the full page and not just the table and are exporting to HTML, save the \code{vtable} to file with \code{file='filename.html'}, and then put the page in your RMarkdown document with \code{<iframe>}.
+#'
+#' @param data Data set; accepts any format with column names. If variable labels are set with the haven package, \code{set_label()} from sjlabelled, or \code{label()} from Hmisc, \code{vtable} will extract them automatically.
+#' @param out Determines where the completed table is sent. Set to \code{"browser"} to open HTML file in browser using \code{browseURL()}, \code{"viewer"} to open in RStudio viewer using \code{viewer()}, if available. Use \code{"htmlreturn"} to return the HTML code to R. Use \code{"return"} to return the completed variable table to R in data frame form or \code{"kable"} to return it as a \code{knitr::kable()}. Additional options include \code{"latex"} for a LaTeX table or \code{"latexpage"} for a full buildable LaTeX page. Defaults to \code{"viewer"} if RStudio is running, \code{"browser"} if it isn't, or \code{"kable"} if it's an RMarkdown document being built with \code{knitr}.
+#' @param file Saves the completed variable table file to HTML or .tex with this filepath. May be combined with any value of \code{out}, although note that \code{out = "return"} and \code{out = "kable"} will still save the standard vtable HTML file as with \code{out = "viewer"} or \code{out = "browser"}.
+#' @param labels Variable labels. labels will accept three formats: (1) A vector of the same length as the number of variables in the data, in the same order as the variables in the data set, (2) A matrix or data frame with two columns and more than one row, where the first column contains variable names (in any order) and the second contains labels, or (3) A matrix or data frame where the column names (in any order) contain variable names and the first row contains labels. Setting the labels parameter will override any variable labels already in the data. Set to \code{"omit"} if the data set has embedded labels but you don't want any labels in the table.
+#' @param class Set to \code{TRUE} to include variable classes in the variable table. Defaults to \code{TRUE}.
+#' @param values Set to \code{TRUE} to include the range of values of each variable: min and max for numeric variables, list of factors for factor or ordered variables, and 'TRUE FALSE' for logicals. values will detect and use value labels set by the sjlabelled or haven packages, as long as every value is labelled. Defaults to \code{TRUE}.
+#' @param missing Set to \code{TRUE} to include the number of NAs in the variable. Defaults to \code{FALSE}.
+#' @param index Set to \code{TRUE} to include the index number of the column with the variable name. Defaults to \code{FALSE}.
+#' @param factor.limit Sets maximum number of factors that will be included if \code{values = TRUE}. Set to 0 for no limit. Defaults to 5.
+#' @param char.values Set to \code{TRUE} to include values of character variables as though they were factors, if \code{values = TRUE}. Or, set to a character vector of variable names to list values of only those character variables. Defaults to \code{FALSE}. Has no effect if \code{values = FALSE}.
 #' @param data.title Character variable with the title of the dataset.
-#' @param desc Character variable offering a brief description of the dataset itself. This will by default include information on the number of observations and the number of columns. To remove this, set desc='omit', or include any description and then include 'omit' as the last four characters.
+#' @param desc Character variable offering a brief description of the dataset itself. This will by default include information on the number of observations and the number of columns. To remove this, set \code{desc='omit'}, or include any description and then include \code{'omit'} as the last four characters.
+#' @param note Table note to go after the last row of the table.
+#' @param anchor Character variable to be used to set an anchor link in HTML tables, or a label tag in LaTeX.
 #' @param col.width Vector of page-width percentages, on 0-100 scale, overriding default column widths in HTML table. Must have a number of elements equal to the number of columns in the resulting table.
-#' @param summ Character vector of summary statistics to include for numeric and logical variables, in the form 'function(x)'. This option is flexible, and allows any summary statistic function that takes in a column and returns a single number. For example, summ=c('mean(x)','mean(log(x))') will provide the mean of each variable as well as the mean of the log of each variable. Keep in mind the special vtable helper functions designed specifically for this option propNA and countNA, which report counts and proportions of NAs in the vectors, and nuniq, which reports the number of unique values. NAs will be omitted from all calculations other than propNA(x) and countNA(x).
-#' @param lush Set to TRUE to select a set of options with more information: sets char.values and missing to TRUE, and sets summ to c('mean(x)', 'sd(x)', 'nuniq(x)'). summ can be overwritten by setting summ to something else.
+#' @param col.align For HTML output, a character vector indicating the HTML \code{text-align} attributes to be used in the table (for example \code{col.align = c('left','center','center')}. Defaults to all left-aligned. If you want to get tricky, you can add a \code{";"} afterwards and keep putting in whatever CSS attributes you want. They will be applied to the whole column.
+#' @param align For LaTeX output, string indicating the alignment of each column. Use standard LaTeX syntax (i.e. \code{l|ccc}). Defaults to all \code{p{}} columns with widths set using the same defaults as with \code{col.width}.  Be sure to escape special characters, in particular backslashes (i.e. \code{p{.25\\\\textwidth}} instead of \code{p{.25\\textwidth}}).
+#' @param summ Character vector of summary statistics to include for numeric and logical variables, in the form \code{'function(x)'}. This option is flexible, and allows any summary statistic function that takes in a column and returns a single number. For example, \code{summ=c('mean(x)','mean(log(x))')} will provide the mean of each variable as well as the mean of the log of each variable. Keep in mind the special vtable package helper functions designed specifically for this option \code{propNA}, \code{countNA}, and \code{notNA}, which report counts and proportions of NAs, or counts of not-NAs, in the vectors, \code{nuniq}, which reports the number of unique values, and \code{pctile}, which returns a vector of the 100 percentiles of the variable. NAs will be omitted from all calculations other than \code{propNA(x)} and \code{countNA(x)}.
+#' @param lush Set to \code{TRUE} to select a set of options with more information: sets \code{char.values} and \code{missing} to \code{TRUE}, and sets summ to \code{c('mean(x)', 'sd(x)', 'nuniq(x)')}. \code{summ} can be overwritten by setting \code{summ} to something else.
+#' @param opts The same \code{vtable} options as above, but in a named list format. Useful for applying the same set of options to multiple \code{vtable}s.
 #' @examples
 #' \dontshow{
 #' #These tests use the out='htmlreturn' option
@@ -115,8 +122,10 @@
 #' @export
 vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=FALSE,
                    index=FALSE,factor.limit=5,char.values=FALSE,
-                   data.title=NA,desc=NA,col.width=NA,summ=NA,lush=FALSE) {
-
+                   data.title=NA,desc=NA,note = NA,anchor=NA,col.width=NA,col.align=NA,
+                   align=NA,summ=NA,lush=FALSE,opts=list()) {
+  #Bring in opts
+  list2env(opts,envir=environment())
   #######CHECK INPUTS
   if (is.null(colnames(data))) {
     stop('Requires data with variable names or column names.')
@@ -148,6 +157,9 @@ vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=
   if (!is.na(desc) & !is.character(desc)) {
     stop('desc must be a character variable.')
   }
+  if (!identical(anchor,NA) & !is.character(anchor)) {
+    stop('anchor must be a character variable.')
+  }
   if (min(is.na(col.width)) == 0 & (!is.vector(col.width) | !is.numeric(col.width) | sum(is.na(col.width)) > 0)) {
     stop('col.width must be a numeric vector with no missing values.')
   }
@@ -160,6 +172,9 @@ vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=
   if (!is.logical(lush)) {
     stop('lush must be logical.')
   }
+  if (!identical(out,NA) & !(out %in% c('viewer', 'browser','return','htmlreturn','kable','latex','latexpage'))) {
+    stop('out must be viewer, browser, return, htmlreturn, kable, latex, or latexpage')
+  }
 
   #One-column matrices run into some problems later on
   if (is.matrix(data) & dim(data)[2] == 1) {
@@ -182,7 +197,7 @@ vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=
   }
 
   ####### FORM VARIABLE TABLE TO BUILD ON
-  #If index = TRUE, start with that. Otherwise, start wtih Name
+  #If index = TRUE, start with that. Otherwise, start with Name
   if (index==TRUE) {
     vt <- data.frame(Index=1:ncol(data),
                      Name=colnames(data))
@@ -470,30 +485,127 @@ vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=
               #4. within each of those variables, paste together a bunch of summary stats
               # Send to parsesumm so as to handle different cases
               function(x) parsesumm(x,summuse,summnames)),sep='')
+  }
 
-    # #Now do all the stats for numeric and logicals. This one's a beast!
-    # #Comments are numbered for the purpose of reading them in order
-    # vt[!(sapply(data,class) %in% c("character","factor")),]$Summary <-
-    #   #8. And finally paste it together with what we already have
-    #   paste(vt[!(sapply(data,class) %in% c("character","factor")),]$Summary,
-    #   #3. Go through each of those variables one by one to calculate summary stats
-    #   sapply(
-    #   #2 Turn it into a list and restrict each of the columns to nonmissing
-    #     #(can't just use complete.cases - you want each variable to have all its nonmissings)
-    #     #(can't use na.rm since some functions might not take it)
-    #     #(can't do this at the level of x on the innermost sapply since it may be difficult to locate the x that connotates data if the function has the letter x in it)
-    #   lapply(as.list(
-    #   #1. Take all the variables that aren't characters or factors
-    #   data[,!(sapply(data,class) %in% c("character","factor")),drop=FALSE]),function(x) x[!is.na(x)]),
-    #   #4. within each of those variables, paste together a bunch of summary stats
-    #   function(x) paste0(
-    #     #5. Each of those summary stats should be preceded by the name of the summary stat
-    #     paste(summnames,
-    #     #6. then, the summary stat itself, for which we need to run through everything in summuse
-    #     sapply(summuse,function(y) round(eval(parse(text=y)),3)),sep=''),
-    #     #7. Bring together with a line break between each summary stat except the last
-    #     collapse='<br/>')),sep='')
+  ####### APPLICATION OF COL.WIDTH OPTION
+  #column percentages
+  #Check which columns we have to account for
+  haslabels <- 'Label' %in% colnames(vt)
+  hasclass <- 'Class' %in% colnames(vt)
+  hasvalues <- 'Values' %in% colnames(vt)
+  hassumm <- 'Summary' %in% colnames(vt)
+  #If col.width not manually set, use defaults
+  if (sum(!is.na(col.width)) == 0) {
+    #initialize vector
+    col.width <- rep(0,length(colnames(vt)))
 
+    #Default ratios:
+    #Name:class:label:values:summ
+    #1:.5:1.75:1.25:.75
+    col.width[colnames(vt)=='Index'] <- .25
+    col.width[colnames(vt)=='Name'] <- 1
+    col.width[colnames(vt)=='Class'] <- .5
+    col.width[colnames(vt)=='Label'] <- 1.75
+    col.width[colnames(vt)=='Values'] <- 1.25
+    col.width[colnames(vt)=='Missing'] <- .5
+    col.width[colnames(vt)=='Summary'] <- .9
+
+    #Add it up
+    totalwidth <- sum(col.width)
+
+    #If total amount is less than two name-spaces, let table take up 60% of screen
+    #From 2-3 name-spaces, 80%
+    #More than 3 is full-screen
+    tablescale <- 60 + 20*(totalwidth>=2) + 20*(totalwidth>=3)
+
+    #And rescale column widths
+    col.width <- (col.width/totalwidth)*tablescale
+
+  }
+
+  #col.align defaults
+  if (identical(col.align, NA)) {
+    col.align <- rep('left',ncol(vt))
+  }
+
+  #Do we have a summary?
+  if ('Summary' %in% names(vt)) {
+    # Don't escape any line breaking stuff in there
+    no.escape <- which(names(vt) == 'Summary')
+  } else {
+    no.escape <- NA
+  }
+
+  ####### LATEX OUTPUT
+  if (!identical(out, NA) & out %in% c('latex','latexpage')) {
+    #Some <br/>s are hardcoded in there
+    if ('Summary' %in% names(vt)) {
+      vt$Summary <- gsub('<br/>',' \\\\\\\\ ',vt$Summary)
+      vt$Summary <- paste0('\\begin{tabular}[c]{@{}c@{}}',vt$Summary,'\\end{tabular}')
+    }
+    if (is.na(align)) {
+      col.width <- col.width/100
+      align <- paste0(paste0('p{',col.width,'\\textwidth}'),collapse='')
+    }
+
+    #Table only
+    if (out == 'latex') {
+      return(cat(dftoLaTeX(vt, file = file,
+                       align = align, note = note,
+                       anchor = anchor, title = 'Variable Table',
+                       no.escape = no.escape)))
+    }
+
+    #Now for the full page
+    out.latex <- '\\documentclass{article}\n\\begin{document}\n\nvtable \\{vtable\\}\n\n'
+    out.latex <- paste(out.latex,
+                       '\\textbf{\\LARGE ', data.title,'}\n\n')
+
+    #Applying description
+    #By default, this is number of obs and number of columns, plus whatever is in desc.
+    #"omit" will leave that out.
+    description <- paste('This data contains ',dim(data)[1],' rows and ',dim(data)[2],' columns.',sep='')
+    #four possibilities: desc is NA (print description),
+    #desc is just omit (print nothing, do nothing)
+    #desc is other stuff followed by omit (just print the other stuff)
+    #desc is other stuff not followed by omit (print desc and the other stuff)
+    #First, check for blank desc
+    if (is.na(desc)) {
+      out.latex <- paste(out.latex,description,'\n\n',sep='')
+
+      #Evaluate these only if desc is nonmissing
+    } else if(desc == "omit") {
+      #Do nothing here
+
+      #Next, stuff followed by omit
+    } else if(substring(desc,nchar(desc)-3)=="omit" & desc != "omit") {
+      #Don't actually print the omit!
+      out.latex <- paste(out.latex,substring(desc,1,nchar(desc)-4),'\n\n',sep='')
+
+      #Finally, stuff not followed by omit
+    } else {
+      out.latex <- paste(out.latex,desc,' ',description,'\n\n',sep='')
+    }
+
+    #And bring in the table itself
+    out.latex <- paste(out.latex,dftoLaTeX(vt, align = align,
+                                           anchor = anchor, note = note,
+                                           title = 'Variable Table',
+                                           no.escape = no.escape),'\n\n\\end{document}',sep='')
+
+    ####### APPLICATION OF FILE OPTION
+    if (!is.na(file)) {
+      #If they forgot a file extension, fill it in
+      if (!grepl("\\.tex",file)) {
+        file <- paste(file,'.tex',sep='')
+      }
+
+      filepath <- file.path(file)
+      #Create temporary tex file
+      writeLines(out.latex,filepath)
+    }
+
+    return(cat(out.latex))
   }
 
 
@@ -572,51 +684,18 @@ vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=
 
   out.html <- paste(out.html,'<h3>Variable Table</h3>',sep='')
 
-  ####### APPLICATION OF COL.WIDTH OPTION
-  #column percentages
-  #Check which columns we have to account for
-  haslabels <- 'Label' %in% colnames(vt)
-  hasclass <- 'Class' %in% colnames(vt)
-  hasvalues <- 'Values' %in% colnames(vt)
-  hassumm <- 'Summary' %in% colnames(vt)
-  #If col.width not manually set, use defaults
-  if (sum(!is.na(col.width)) == 0) {
-    #initialize vector
-    col.width <- rep(0,length(colnames(vt)))
-
-    #Default ratios:
-    #Name:class:label:values:summ
-    #1:.5:1.75:1.25:.75
-    col.width[colnames(vt)=='Index'] <- .25
-    col.width[colnames(vt)=='Name'] <- 1
-    col.width[colnames(vt)=='Class'] <- .5
-    col.width[colnames(vt)=='Label'] <- 1.75
-    col.width[colnames(vt)=='Values'] <- 1.25
-    col.width[colnames(vt)=='Missing'] <- .5
-    col.width[colnames(vt)=='Summary'] <- .9
-
-    #Add it up
-    totalwidth <- sum(col.width)
-
-    #If total amount is less than two name-spaces, let table take up 60% of screen
-    #From 2-3 name-spaces, 80%
-    #More than 3 is full-screen
-    tablescale <- 60 + 20*(totalwidth>=2) + 20*(totalwidth>=3)
-
-    #And rescale column widths
-    col.width <- (col.width/totalwidth)*tablescale
-
-  }
-
-
   #And bring in the table itself
-  out.html <- paste(out.html,dftoHTML(vt,out='htmlreturn',col.width=col.width),'</body></html>',sep='')
+  out.html <- paste(out.html,dftoHTML(vt,out='htmlreturn',
+                                      col.width=col.width,
+                                      col.align=col.align,
+                                      note = note, anchor=anchor,
+                                      no.escape = no.escape),'</body></html>',sep='')
 
 
   ####### APPLICATION OF FILE OPTION
   if (!is.na(file)) {
     #If they forgot a file extension, fill it in
-    if (!grepl("\\.",file)) {
+    if (!grepl("\\.htm",file)) {
       file <- paste(file,'.html',sep='')
     }
 
@@ -645,18 +724,29 @@ vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=
 
   #Either print the variable table to the help window
   #or return a variable table to the screen, as desired
-  if (Sys.getenv('RSTUDIO')=='1' & (out == 'viewer' | out == '')) {
+  #Default to kable if in knitr
+  if (out == 'kable' | (isTRUE(getOption('knitr.in.progress')) & out == '')) {
+    for (i in 1:ncol(vt)) {
+      vt[[i]] <- gsub('<br/>','<br>',vt[[i]])
+    }
+    #kable can't handle blank rows. These should not occur in vtable but just in case
+    vt <- vt[!apply(vt,MARGIN=1,FUN=function(x) !any(!(x==rep('',ncol(vt))))),]
+    #I don't know how this would happen but just in case
+    vt <- vt[!apply(vt,MARGIN=1,FUN=function(x) propNA(x) == 1),]
+    return(knitr::kable(vt))
+  } else if (Sys.getenv('RSTUDIO')=='1' & (out == 'viewer' | out == '')) {
     rstudioapi::viewer(htmlpath)
   } else if (Sys.getenv('RSTUDIO')=='' & out == 'viewer') {
-    stop('out = viewer is not a valid option if RStudio is not running.')
+    stop('out = "viewer" is not a valid option if RStudio is not running.')
   } else if ((Sys.getenv('RSTUDIO')=='' & out == '') | (out == 'browser')) {
     utils::browseURL(htmlpath)
   } else if (out == 'return') {
+    for (i in 1:ncol(vt)) {
+      vt[[i]] <- gsub('<br/>','; ',vt[[i]])
+    }
     return(vt)
   } else if (out == 'htmlreturn') {
-    return(out.html)
-  } else {
-    stop('Unrecognized value of out. Set to \"viewer\", \"browser\", \"return\", \"htmlreturn\", or leave blank.')
+    return(cat(out.html))
   }
 }
 
@@ -665,145 +755,3 @@ vtable <- function(data,out=NA,file=NA,labels=NA,class=TRUE,values=TRUE,missing=
 vt <- vtable
 
 
-#' Data Frame to HTML Function
-#'
-#' This function takes a data frame or matrix with column names and outputs an HTML table version of that data frame.
-#'
-#' This function is designed to feed HTML versions of variable tables to vtable() and labeltable().
-#'
-#' @param data Data set; accepts any format with column names.
-#' @param out Determines where the completed table is sent. Set to "browser" to open HTML file in browser using browseURL(), "viewer" to open in RStudio viewer using viewer(), if available, or "htmlreturn" to return the HTML code. Defaults to Defaults to "viewer" if RStudio is running and "browser" if it isn't.
-#' @param file Saves the completed variable table file to HTML with this filepath. May be combined with any value of out.
-#' @param col.width Vector of page-width percentages, on 0-100 scale, overriding default column widths in HTML table. Must have a number of elements equal to the number of columns in the resulting table.
-#' @param row.names Flag determining whether or not the row names should be included in the table. Defaults to FALSE.
-#' @examples
-#' df <- data.frame(var1 = 1:4,var2=5:8,var3=c('A','B','C','D'),
-#'     var4=as.factor(c('A','B','C','C')),var5=c(TRUE,TRUE,FALSE,FALSE))
-#' dftoHTML(df,out="browser")
-#'
-
-#' @export
-dftoHTML <- function(data,out=NA,file=NA,col.width=NA,row.names=FALSE) {
-  if (is.null(colnames(data))) {
-    stop('Requires data with variable names or column names.')
-  }
-  if (!is.na(file) & !is.character(file)) {
-    stop('Incorrect file name.')
-  }
-  if (min(is.na(col.width)) == 0 & (!is.vector(col.width) | !is.numeric(col.width) | sum(is.na(col.width)) > 0)) {
-    stop('col.width must be a numeric vector with no missing values.')
-  }
-  if (min(is.na(col.width)) == 0 & (max(col.width) > 100 | min(col.width) < 0)) {
-    stop('Elements of col.width must be between 0 and 100.')
-  }
-  if (!is.logical(row.names)) {
-    stop('The row.names option must be TRUE or FALSE.')
-  }
-
-  #If row.names = TRUE, the row names must be included as their own column
-  if (row.names==TRUE) {
-    data <- cbind(row.names(data),data)
-    names(data)[1] <- "row.names"
-  }
-
-  #Get the column headers
-  heads <- colnames(data)
-
-  #Set default column widths
-  if (sum(!is.na(col.width))==0) {
-    col.width = rep(100/dim(data)[2],dim(data)[2])
-  }
-
-  #Turn column widths to rounded characters
-  col.width <- as.character(round(as.numeric(col.width,2)))
-
-  #Put together column widths with the column titles. First, create <th style = "width:pct%"> tags
-  pctsh <- sapply(col.width,function(x) paste('<th style=\"width:',x,'%\">',sep=""))
-  #Then, combine those tags with the column headers
-  heads <- apply(cbind(pctsh,heads),1,paste0,collapse='')
-
-  #Begin table html with a row of table headers, finishing off those '<th>Name' strings with '</td>'
-  table.html <- paste('<table><tr>',
-                    paste0(sapply(heads,function(x) paste(x,'</th>',sep='')),collapse=''),
-                    '</tr>',sep='')
-
-  #Convert rows of data to html
-  #Put together column percentages with the td's, getting <td style = "width:pct%"> tags
-  pctsd <- sapply(col.width,function(x) paste('<td style=\"width:',x,'%\">',sep=""))
-
-  #Alright! This command.
-  #The outer apply goes through each row of the data frame and sends it to the inner apply
-  #The inner apply combines the <td> opener tags with the data in that cell using cbind(),
-  #then collapses it all together so you get a single string <td>DATA.
-  #Then t() transposes things back to how we need them
-  #Split it based on the number of columns because if there's only one column,
-  #the double-apply will squish it too far into just one row!
-  if (dim(data)[2]==1) {
-    vtablerows <- paste(pctsd[1],data[,1],sep="")
-  } else{
-    vtablerows <- t(apply(data,1,function(y) apply(cbind(pctsd,y),1,paste0,collapse='')))
-  }
-  #Then, this one starts with the inner sapply combining <td>DATA with </td> so we have <td>DATA</td>
-  #Then, the outer apply takes each of those <td>DATA</td>s and slaps them all together
-  #into a single string
-  #Again, separate by number of columns to avoid over-smushing
-  if (dim(data)[2]==1) {
-    vtablerows <- paste(vtablerows,"</td>",sep='')
-  } else{
-    vtablerows <- t(apply(vtablerows,1,function(y)
-      paste0(sapply(y,function(x) paste(x,'</td>',sep='')),collapse='')))
-  }
-  #Then finally, we take those <td>DATA</td><td>DATA</td> strings, wrap them
-  #in <tr> and </tr>, and finally stick them all together to make the bulk of our table
-  table.html <- paste(table.html,
-                    paste0(sapply(vtablerows,function(x)
-                      paste('<tr>',x,'</tr>',sep='')),collapse=''))
-  #And close out the table
-  table.html <- paste(table.html,'</table>',sep='')
-
-  ####### APPLICATION OF FILE OPTION
-  if (!is.na(file)) {
-    #If they forgot a file extension, fill it in
-    if (!grepl("\\.",file)) {
-      file <- paste(file,'.html',sep='')
-    }
-
-    filepath <- file.path(file)
-    #Create temporary html file
-    writeLines(table.html,filepath)
-  }
-
-  #For better evaluating if statements
-  if (is.na(out)) {
-    out = ''
-  }
-
-  ####### APPLICATION OF OUT OPTION
-  #If the plan is to produce a viewable HTML, create it
-  if (out == 'viewer' | out == 'browser' | out == '') {
-    #Get temporary dirpath
-    tempDir <- tempfile()
-    #Create temporary directory
-    dir.create(tempDir)
-    #Get temporary filepath
-    htmlpath <- file.path(tempDir,'dftoHTML.html')
-    #Create temporary html file
-    writeLines(table.html,htmlpath)
-  }
-
-  #Either print the variable table to the help window
-  #or return a variable table to the screen, as desired
-
-  if (Sys.getenv('RSTUDIO')=='1' & (out == 'viewer' | out == '')) {
-    rstudioapi::viewer(htmlpath)
-  } else if (Sys.getenv('RSTUDIO')=='' & out == 'viewer') {
-    stop('out = viewer is not a valid option if RStudio is not running.')
-  } else if ((Sys.getenv('RSTUDIO')=='' & out == '') | (out == 'browser')) {
-    utils::browseURL(htmlpath)
-  } else if (out == 'htmlreturn') {
-    return(table.html)
-  } else {
-    stop('Unrecognized value of out. Set to \"viewer\", \"browser\", \"htmlreturn\", or leave blank.')
-  }
-
-}
